<?php

/* ***********************************************************
 * This file was automatically generated on 2012-06-28.      *
 *                                                           *
 * If you have a bugfix for this file and want to commit it, *
 * please fix the bug in the generator. You can find a link  *
 * to the generator git on tinkerforge.com                   *
 *************************************************************/

namespace Tinkerforge;

require_once(__DIR__ . '/IPConnection.php');

/**
 * Device for controlling stepper motors
 */
class BrickStepper extends Device
{

    /**
     * This callback is triggered when the input voltage drops below the value set by
     * BrickStepper::setMinimumVoltage(). The parameter is the current voltage given
     * in mV.
     */
    const CALLBACK_UNDER_VOLTAGE = 31;

    /**
     * This callback is triggered when a position set by BrickStepper::setSteps() or
     * BrickStepper::setTargetPosition() is reached.
     * 
     * <note>
     *  Since we can't get any feedback from the stepper motor, this only works if the
     *  acceleration (see BrickStepper::setSpeedRamping()) is set smaller or equal to the
     *  maximum acceleration of the motor. Otherwise the motor will lag behind the
     *  control value and the callback will be triggered too early.
     * </note>
     */
    const CALLBACK_POSITION_REACHED = 32;


    /**
     * @internal
     */
    const FUNCTION_SET_MAX_VELOCITY = 1;

    /**
     * @internal
     */
    const FUNCTION_GET_MAX_VELOCITY = 2;

    /**
     * @internal
     */
    const FUNCTION_GET_CURRENT_VELOCITY = 3;

    /**
     * @internal
     */
    const FUNCTION_SET_SPEED_RAMPING = 4;

    /**
     * @internal
     */
    const FUNCTION_GET_SPEED_RAMPING = 5;

    /**
     * @internal
     */
    const FUNCTION_FULL_BRAKE = 6;

    /**
     * @internal
     */
    const FUNCTION_SET_CURRENT_POSITION = 7;

    /**
     * @internal
     */
    const FUNCTION_GET_CURRENT_POSITION = 8;

    /**
     * @internal
     */
    const FUNCTION_SET_TARGET_POSITION = 9;

    /**
     * @internal
     */
    const FUNCTION_GET_TARGET_POSITION = 10;

    /**
     * @internal
     */
    const FUNCTION_SET_STEPS = 11;

    /**
     * @internal
     */
    const FUNCTION_GET_STEPS = 12;

    /**
     * @internal
     */
    const FUNCTION_GET_REMAINING_STEPS = 13;

    /**
     * @internal
     */
    const FUNCTION_SET_STEP_MODE = 14;

    /**
     * @internal
     */
    const FUNCTION_GET_STEP_MODE = 15;

    /**
     * @internal
     */
    const FUNCTION_DRIVE_FORWARD = 16;

    /**
     * @internal
     */
    const FUNCTION_DRIVE_BACKWARD = 17;

    /**
     * @internal
     */
    const FUNCTION_STOP = 18;

    /**
     * @internal
     */
    const FUNCTION_GET_STACK_INPUT_VOLTAGE = 19;

    /**
     * @internal
     */
    const FUNCTION_GET_EXTERNAL_INPUT_VOLTAGE = 20;

    /**
     * @internal
     */
    const FUNCTION_GET_CURRENT_CONSUMPTION = 21;

    /**
     * @internal
     */
    const FUNCTION_SET_MOTOR_CURRENT = 22;

    /**
     * @internal
     */
    const FUNCTION_GET_MOTOR_CURRENT = 23;

    /**
     * @internal
     */
    const FUNCTION_ENABLE = 24;

    /**
     * @internal
     */
    const FUNCTION_DISABLE = 25;

    /**
     * @internal
     */
    const FUNCTION_IS_ENABLED = 26;

    /**
     * @internal
     */
    const FUNCTION_SET_DECAY = 27;

    /**
     * @internal
     */
    const FUNCTION_GET_DECAY = 28;

    /**
     * @internal
     */
    const FUNCTION_SET_MINIMUM_VOLTAGE = 29;

    /**
     * @internal
     */
    const FUNCTION_GET_MINIMUM_VOLTAGE = 30;

    /**
     * @internal
     */
    const FUNCTION_SET_SYNC_RECT = 33;

    /**
     * @internal
     */
    const FUNCTION_IS_SYNC_RECT = 34;

    /**
     * Creates an object with the unique device ID $uid. This object can
     * then be added to the IP connection.
     *
     * @param string $uid
     */
    public function __construct($uid)
    {
        parent::__construct($uid);

        $this->expectedName = 'Stepper Brick';

        $this->bindingVersion = array(1, 0, 0);

        $this->callbackWrappers[self::CALLBACK_UNDER_VOLTAGE] = 'callbackWrapperUnderVoltage';
        $this->callbackWrappers[self::CALLBACK_POSITION_REACHED] = 'callbackWrapperPositionReached';
    }

    /**
     * @internal
     * @param string $header
     * @param string $data
     */
    public function handleCallback($header, $data)
    {
        call_user_func(array($this, $this->callbackWrappers[$header['functionID']]), $data);
    }

    /**
     * Sets the maximum velocity of the stepper motor in steps per second.
     * This function does *not* start the motor, it merely sets the maximum
     * velocity the stepper motor is accelerated to. To get the motor running use
     * either BrickStepper::setTargetPosition(), BrickStepper::setSteps(), BrickStepper::driveForward() or
     * BrickStepper::driveBackward().
     * 
     * @param int $velocity
     * 
     * @return void
     */
    public function setMaxVelocity($velocity)
    {
        $payload = '';
        $payload .= pack('v', $velocity);

        $this->sendRequestNoResponse(self::FUNCTION_SET_MAX_VELOCITY, $payload);
    }

    /**
     * Returns the velocity as set by BrickStepper::setMaxVelocity().
     * 
     * 
     * @return int
     */
    public function getMaxVelocity()
    {
        $payload = '';

        $data = $this->sendRequestExpectResponse(self::FUNCTION_GET_MAX_VELOCITY, $payload, 2);

        $payload = unpack('v1velocity', $data);

        return $payload['velocity'];
    }

    /**
     * Returns the *current* velocity of the stepper motor in steps per second.
     * 
     * 
     * @return int
     */
    public function getCurrentVelocity()
    {
        $payload = '';

        $data = $this->sendRequestExpectResponse(self::FUNCTION_GET_CURRENT_VELOCITY, $payload, 2);

        $payload = unpack('v1velocity', $data);

        return $payload['velocity'];
    }

    /**
     * Sets the acceleration and deacceleration of the stepper motor. The values
     * are given in *steps/s²*. An acceleration of 1000 means, that
     * every second the velocity is increased by 1000 *steps/s*.
     * 
     * For example: If the current velocity is 0 and you want to accelerate to a
     * velocity of 8000 *steps/s* in 10 seconds, you should set an acceleration
     * of 800 *steps/s²*.
     * 
     * An acceleration/deacceleration of 0 means instantaneous 
     * acceleration/deacceleration (not recommended)
     * 
     * The default value is 1000 for both
     * 
     * @param int $acceleration
     * @param int $deacceleration
     * 
     * @return void
     */
    public function setSpeedRamping($acceleration, $deacceleration)
    {
        $payload = '';
        $payload .= pack('v', $acceleration);
        $payload .= pack('v', $deacceleration);

        $this->sendRequestNoResponse(self::FUNCTION_SET_SPEED_RAMPING, $payload);
    }

    /**
     * Returns the acceleration and deacceleration as set by 
     * BrickStepper::setSpeedRamping().
     * 
     * 
     * @return array
     */
    public function getSpeedRamping()
    {
        $result = array();

        $payload = '';

        $data = $this->sendRequestExpectResponse(self::FUNCTION_GET_SPEED_RAMPING, $payload, 4);

        $payload = unpack('v1acceleration/v1deacceleration', $data);

        $result['acceleration'] = $payload['acceleration'];
        $result['deacceleration'] = $payload['deacceleration'];

        return $result;
    }

    /**
     * Executes an active full brake. 
     *  
     * <warning>
     *   This function is for emergency purposes,
     *   where an immediate brake is necessary. Depending on the current velocity and
     *   the strength of the motor, a full brake can be quite violent.
     * </warning>
     * 
     * Call BrickStepper::stop() if you just want to stop the motor.
     * 
     * 
     * @return void
     */
    public function fullBrake()
    {
        $payload = '';

        $this->sendRequestNoResponse(self::FUNCTION_FULL_BRAKE, $payload);
    }

    /**
     * Sets the current steps of the internal step counter. This can be used to
     * set the current position to 0 when some kind of starting position
     * is reached (e.g. when a CNC machine reaches a corner).
     * 
     * @param int $position
     * 
     * @return void
     */
    public function setCurrentPosition($position)
    {
        $payload = '';
        $payload .= pack('V', $position);

        $this->sendRequestNoResponse(self::FUNCTION_SET_CURRENT_POSITION, $payload);
    }

    /**
     * Returns the current position of the stepper motor in steps. On startup
     * the position is 0. The steps are counted with all possible driving
     * functions (BrickStepper::setTargetPosition(), BrickStepper::setSteps(), BrickStepper::driveForward() or
     * BrickStepper::driveBackward()). It also is possible to reset the steps to 0 or
     * set them to any other desired value with BrickStepper::setCurrentPosition().
     * 
     * 
     * @return int
     */
    public function getCurrentPosition()
    {
        $payload = '';

        $data = $this->sendRequestExpectResponse(self::FUNCTION_GET_CURRENT_POSITION, $payload, 4);

        $payload = unpack('V1position', $data);

        return IPConnection::fixUnpackedInt32($payload['position']);
    }

    /**
     * Sets the target position of the stepper motor in steps. For example,
     * if the current position of the motor is 500 and BrickStepper::setTargetPosition() is
     * called with 1000, the stepper motor will drive 500 steps forward. It will
     * use the velocity, acceleration and deacceleration as set by
     * BrickStepper::setMaxVelocity() and BrickStepper::setSpeedRamping().
     * 
     * A call of BrickStepper::setTargetPosition() with the parameter *x* is equivalent to
     * a call of BrickStepper::setSteps() with the parameter 
     * (*x* - BrickStepper::getCurrentPosition()).
     * 
     * @param int $position
     * 
     * @return void
     */
    public function setTargetPosition($position)
    {
        $payload = '';
        $payload .= pack('V', $position);

        $this->sendRequestNoResponse(self::FUNCTION_SET_TARGET_POSITION, $payload);
    }

    /**
     * Returns the last target position as set by BrickStepper::setTargetPosition().
     * 
     * 
     * @return int
     */
    public function getTargetPosition()
    {
        $payload = '';

        $data = $this->sendRequestExpectResponse(self::FUNCTION_GET_TARGET_POSITION, $payload, 4);

        $payload = unpack('V1position', $data);

        return IPConnection::fixUnpackedInt32($payload['position']);
    }

    /**
     * Sets the number of steps the stepper motor should run. Positive values
     * will drive the motor forward and negative values backward. 
     * The velocity, acceleration and deacceleration as set by
     * BrickStepper::setMaxVelocity() and BrickStepper::setSpeedRamping() will be used.
     * 
     * @param int $steps
     * 
     * @return void
     */
    public function setSteps($steps)
    {
        $payload = '';
        $payload .= pack('V', $steps);

        $this->sendRequestNoResponse(self::FUNCTION_SET_STEPS, $payload);
    }

    /**
     * Returns the last steps as set by BrickStepper::setSteps().
     * 
     * 
     * @return int
     */
    public function getSteps()
    {
        $payload = '';

        $data = $this->sendRequestExpectResponse(self::FUNCTION_GET_STEPS, $payload, 4);

        $payload = unpack('V1steps', $data);

        return IPConnection::fixUnpackedInt32($payload['steps']);
    }

    /**
     * Returns the remaining steps of the last call of BrickStepper::setSteps().
     * For example, if BrickStepper::setSteps() is called with 2000 and 
     * BrickStepper::getRemainingSteps() is called after the motor has run for 500 steps,
     * it will return 1500.
     * 
     * 
     * @return int
     */
    public function getRemainingSteps()
    {
        $payload = '';

        $data = $this->sendRequestExpectResponse(self::FUNCTION_GET_REMAINING_STEPS, $payload, 4);

        $payload = unpack('V1steps', $data);

        return IPConnection::fixUnpackedInt32($payload['steps']);
    }

    /**
     * Sets the step mode of the stepper motor. Possible values are:
     * 
     * * Full Step = 1
     * * Half Step = 2
     * * Quarter Step = 4
     * * Eighth Step = 8
     * 
     * A higher value will increase the resolution and
     * decrease the torque of the stepper motor.
     * 
     * The default value is 8 (Eighth Step).
     * 
     * @param int $mode
     * 
     * @return void
     */
    public function setStepMode($mode)
    {
        $payload = '';
        $payload .= pack('C', $mode);

        $this->sendRequestNoResponse(self::FUNCTION_SET_STEP_MODE, $payload);
    }

    /**
     * Returns the step mode as set by BrickStepper::setStepMode().
     * 
     * 
     * @return int
     */
    public function getStepMode()
    {
        $payload = '';

        $data = $this->sendRequestExpectResponse(self::FUNCTION_GET_STEP_MODE, $payload, 1);

        $payload = unpack('C1mode', $data);

        return $payload['mode'];
    }

    /**
     * Drives the stepper motor forward until BrickStepper::driveBackward() or
     * BrickStepper::stop() is called. The velocity, acceleration and deacceleration as 
     * set by BrickStepper::setMaxVelocity() and BrickStepper::setSpeedRamping() will be used.
     * 
     * 
     * @return void
     */
    public function driveForward()
    {
        $payload = '';

        $this->sendRequestNoResponse(self::FUNCTION_DRIVE_FORWARD, $payload);
    }

    /**
     * Drives the stepper motor backward until BrickStepper::driveForward() or
     * BrickStepper::stop() is triggered. The velocity, acceleration and deacceleration as
     * set by BrickStepper::setMaxVelocity() and BrickStepper::setSpeedRamping() will be used.
     * 
     * 
     * @return void
     */
    public function driveBackward()
    {
        $payload = '';

        $this->sendRequestNoResponse(self::FUNCTION_DRIVE_BACKWARD, $payload);
    }

    /**
     * Stops the stepper motor with the deacceleration as set by 
     * BrickStepper::setSpeedRamping().
     * 
     * 
     * @return void
     */
    public function stop()
    {
        $payload = '';

        $this->sendRequestNoResponse(self::FUNCTION_STOP, $payload);
    }

    /**
     * Returns the stack input voltage in mV. The stack input voltage is the
     * voltage that is supplied via the stack, i.e. it is given by a 
     * Step-Down or Step-Up Power Supply.
     * 
     * 
     * @return int
     */
    public function getStackInputVoltage()
    {
        $payload = '';

        $data = $this->sendRequestExpectResponse(self::FUNCTION_GET_STACK_INPUT_VOLTAGE, $payload, 2);

        $payload = unpack('v1voltage', $data);

        return $payload['voltage'];
    }

    /**
     * Returns the external input voltage in mV. The external input voltage is
     * given via the black power input connector on the Stepper Brick. 
     *  
     * If there is an external input voltage and a stack input voltage, the motor
     * will be driven by the external input voltage. If there is only a stack 
     * voltage present, the motor will be driven by this voltage.
     * 
     * <warning>
     *   This means, if you have a high stack voltage and a low external voltage,
     *   the motor will be driven with the low external voltage. If you then remove
     *   the external connection, it will immediately be driven by the high
     *   stack voltage
     * </warning>
     * 
     * 
     * @return int
     */
    public function getExternalInputVoltage()
    {
        $payload = '';

        $data = $this->sendRequestExpectResponse(self::FUNCTION_GET_EXTERNAL_INPUT_VOLTAGE, $payload, 2);

        $payload = unpack('v1voltage', $data);

        return $payload['voltage'];
    }

    /**
     * Returns the current consumption of the motor in mA.
     * 
     * 
     * @return int
     */
    public function getCurrentConsumption()
    {
        $payload = '';

        $data = $this->sendRequestExpectResponse(self::FUNCTION_GET_CURRENT_CONSUMPTION, $payload, 2);

        $payload = unpack('v1current', $data);

        return $payload['current'];
    }

    /**
     * Sets the current in mA with which the motor will be driven.
     * The minimum value is 100mA, the maximum value 2291mA and the 
     * default value is 800mA.
     * 
     * <warning>
     *   Do not set this value above the specifications of your stepper motor.
     *   Otherwise it may damage your motor.
     * </warning>
     * 
     * @param int $current
     * 
     * @return void
     */
    public function setMotorCurrent($current)
    {
        $payload = '';
        $payload .= pack('v', $current);

        $this->sendRequestNoResponse(self::FUNCTION_SET_MOTOR_CURRENT, $payload);
    }

    /**
     * Returns the current as set by BrickStepper::setMotorCurrent().
     * 
     * 
     * @return int
     */
    public function getMotorCurrent()
    {
        $payload = '';

        $data = $this->sendRequestExpectResponse(self::FUNCTION_GET_MOTOR_CURRENT, $payload, 2);

        $payload = unpack('v1current', $data);

        return $payload['current'];
    }

    /**
     * Enables the motor. The motor can be configured (maximum velocity, 
     * acceleration, etc) before it is enabled.
     * 
     * 
     * @return void
     */
    public function enable()
    {
        $payload = '';

        $this->sendRequestNoResponse(self::FUNCTION_ENABLE, $payload);
    }

    /**
     * Disables the motor. The configurations are kept (maximum velocity, 
     * acceleration, etc) but the motor is not driven until it is enabled again.
     * 
     * 
     * @return void
     */
    public function disable()
    {
        $payload = '';

        $this->sendRequestNoResponse(self::FUNCTION_DISABLE, $payload);
    }

    /**
     * Returns true if the motor is enabled, false otherwise.
     * 
     * 
     * @return bool
     */
    public function isEnabled()
    {
        $payload = '';

        $data = $this->sendRequestExpectResponse(self::FUNCTION_IS_ENABLED, $payload, 1);

        $payload = unpack('C1enabled', $data);

        return (bool)$payload['enabled'];
    }

    /**
     * Sets the decay mode of the stepper motor. The possible value range is
     * between 0 and 65535. A value of 0 sets the fast decay mode, a value of
     * 65535 sets the slow decay mode and a value in between sets the mixed
     * decay mode.
     * 
     * Changing the decay mode is only possible if synchronous rectification
     * is enabled (see BrickStepper::setSyncRect()).
     * 
     * For a good explanation of the different decay modes see 
     * `this <http://robot.avayanex.com/?p=86/>`_ blog post by Avayan.
     * 
     * A good decay mode is unfortunately different for every motor. The best
     * way to work out a good decay mode for your stepper motor, if you can't
     * measure the current with an oscilloscope, is to listen to the sound of
     * the motor. If the value is too low, you often hear a high pitched 
     * sound and if it is too high you can often hear a humming sound.
     * 
     * Generally, fast decay mode (small value) will be noisier but also
     * allow higher motor speeds.
     * 
     * The default value is 10000.
     * <note>
     *   There is unfortunately no formula to calculate a perfect decay
     *   mode for a given stepper motor. If you have problems with loud noises
     *   or the maximum motor speed is too slow, you should try to tinker with
     *   the decay value
     * </note>
     * 
     * @param int $decay
     * 
     * @return void
     */
    public function setDecay($decay)
    {
        $payload = '';
        $payload .= pack('v', $decay);

        $this->sendRequestNoResponse(self::FUNCTION_SET_DECAY, $payload);
    }

    /**
     * Returns the decay mode as set by BrickStepper::setDecay()
     * 
     * 
     * @return int
     */
    public function getDecay()
    {
        $payload = '';

        $data = $this->sendRequestExpectResponse(self::FUNCTION_GET_DECAY, $payload, 2);

        $payload = unpack('v1decay', $data);

        return $payload['decay'];
    }

    /**
     * Sets the minimum voltage in mV, below which the BrickStepper::CALLBACK_UNDER_VOLTAGE callback
     * is triggered. The minimum possible value that works with the Stepper Brick is 8V.
     * You can use this function to detect the discharge of a battery that is used
     * to drive the stepper motor. If you have a fixed power supply, you likely do 
     * not need this functionality.
     * 
     * The default value is 8V.
     * 
     * @param int $voltage
     * 
     * @return void
     */
    public function setMinimumVoltage($voltage)
    {
        $payload = '';
        $payload .= pack('v', $voltage);

        $this->sendRequestNoResponse(self::FUNCTION_SET_MINIMUM_VOLTAGE, $payload);
    }

    /**
     * Returns the minimum voltage as set by BrickStepper::setMinimumVoltage().
     * 
     * 
     * @return int
     */
    public function getMinimumVoltage()
    {
        $payload = '';

        $data = $this->sendRequestExpectResponse(self::FUNCTION_GET_MINIMUM_VOLTAGE, $payload, 2);

        $payload = unpack('v1voltage', $data);

        return $payload['voltage'];
    }

    /**
     * Turns synchronous rectification on or off (true/false).
     * 
     * With synchronous rectification on, the decay can be changed
     * (see BrickStepper::setDecay()). Without synchronous rectification fast
     * decay is used.
     * 
     * For an explanation of synchronous rectification see 
     * `here <http://en.wikipedia.org/wiki/Active_rectification>`__.
     * 
     * <warning>
     *  If you want to use high speeds (> 10000 steps/s) for a large 
     *  stepper motor with a large inductivity we strongly
     *  suggest that you disable synchronous rectification. Otherwise the
     *  Brick may not be able to cope with the load and overheat.
     * </warning>
     * 
     * The default value is false.
     * 
     * @param bool $sync_rect
     * 
     * @return void
     */
    public function setSyncRect($sync_rect)
    {
        $payload = '';
        $payload .= pack('C', intval((bool)$sync_rect));

        $this->sendRequestNoResponse(self::FUNCTION_SET_SYNC_RECT, $payload);
    }

    /**
     * Returns true if synchronous rectification is enabled, false otherwise.
     * 
     * 
     * @return bool
     */
    public function isSyncRect()
    {
        $payload = '';

        $data = $this->sendRequestExpectResponse(self::FUNCTION_IS_SYNC_RECT, $payload, 1);

        $payload = unpack('C1sync_rect', $data);

        return (bool)$payload['sync_rect'];
    }

    /**
     * Registers a callback with ID $id to the callable $callback.
     *
     * @param int $id
     * @param callable $callback
     *
     * @return void
     */
    public function registerCallback($id, $callback)
    {
        $this->registeredCallbacks[$id] = $callback;
    }

    /**
     * @internal
     * @param string $data
     */
    public function callbackWrapperUnderVoltage($data)
    {
        $result = array();
        $payload = unpack('v1voltage', $data);

        array_push($result, $payload['voltage']);

        call_user_func_array($this->registeredCallbacks[self::CALLBACK_UNDER_VOLTAGE], $result);
    }

    /**
     * @internal
     * @param string $data
     */
    public function callbackWrapperPositionReached($data)
    {
        $result = array();
        $payload = unpack('V1position', $data);

        array_push($result, IPConnection::fixUnpackedInt32($payload['position']));

        call_user_func_array($this->registeredCallbacks[self::CALLBACK_POSITION_REACHED], $result);
    }
}

?>
